@@ -3,6 +3,7 @@ import json
 import gazu
 import getpass
 import pathlib
+import fnmatch
 
 from kabaret import flow
 from kabaret.app.ui.gui.widgets.flow.flow_view import (
@@ -32,6 +33,7 @@ from .site import (
     WorkingSites, ExchangeSites,
     MinioFileUploader, MinioFileDownloader,
 )
+from .file import TrackedFile, Revision
 from .kitsu import KitsuAPIWrapper
 
 
@@ -177,8 +179,8 @@ class WorkingSiteChoiceValue(flow.values.ChoiceValue):
 
 class RequestRevisionsAs(flow.Action):
 
-    path = flow.SessionParam("").ui(
-        placeholder="Revision oid pattern (e.g., /siren/films/siren/sequences/sq27/shots/*/departments/misc/files/*/history/revisions/v001)"
+    pattern = flow.SessionParam("").watched().ui(
+        placeholder="Revision oid pattern"
     )
     requesting_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
         label="Requesting site"
@@ -186,6 +188,11 @@ class RequestRevisionsAs(flow.Action):
     requested_site_name = flow.Param("default", WorkingSiteChoiceValue).watched().ui(
         label="Site to query"
     )
+    revision_oids = flow.SessionParam("").ui(
+        editor="textarea",
+        html=True,
+        editable=False,
+    )
 
     def allow_context(self, context):
         current_site = self.root().project().get_current_site()
@@ -202,89 +209,90 @@ class RequestRevisionsAs(flow.Action):
                 )
 
             self.message.set(msg)
+        elif child_value is self.pattern:
+            self._oids = self.glob(self.root().project().oid(), self.pattern.get(), 0)
+            self.revision_oids.set("<br>".join(self._oids))
 
     def get_buttons(self):
         self.requested_site_name.set(self.requested_site_name.choices()[0])
+        self.revision_oids.set("")
+        self._oids = None
+
         return ["Request", "Close"]
     
+    def ls(self, root_oid):
+        related_info, mapped_names = self.root().session().cmds.Flow.ls(root_oid)
+        relation_oids = [rel_info[0] for rel_info in related_info]
+        mapped_oids = ["%s/%s" % (root_oid, name) for name in mapped_names]
+        
+        return relation_oids + mapped_oids
+    
+    def get_last_publication(self, file_oid):
+        o = self.root().get_object(file_oid)
+
+        if not isinstance(o, TrackedFile):
+            return None
+        
+        head = o.get_head_revision()
+        
+        if not head:
+            return None
+        
+        return head.name()
+    
+    def glob(self, root_oid, pattern, level):
+        if level >= pattern.count("/") - 1:
+            return [root_oid]
+
+        matches = []
+        level_pattern = "/".join(pattern.split("/")[:level + 3])
+
+        if level_pattern.endswith("[last]"):
+            file_oid = self.root().session().cmds.Flow.resolve_path(root_oid + "/../..")
+            head_name = self.get_last_publication(file_oid)
+
+            if not head_name:
+                return []
+            
+            pattern = pattern.replace("[last]", head_name)
+            level_pattern = level_pattern.replace("[last]", head_name)
+        
+        for oid in self.ls(root_oid):
+            if fnmatch.fnmatch(oid, level_pattern):
+                matches += self.glob(oid, pattern, level + 1)
+        
+        return matches
+    
     def run(self, button):
         if button == "Close":
             return
         
-        requested_site_name = self.requested_site_name.get()
-        requesting_site_name = self.requesting_site_name.get()
-        
-        if requested_site_name == requesting_site_name:
-            return self.get_result(close=False)
-        
-        self.message.set("<h2>Request revisions</h2>")
-        
-        splitted_oids = self.path.get().split('*')
-        lvl = len(splitted_oids)
-        oids = []
+        if self._oids is None:
+            self._oids = self.glob(self.root().project().oid(), self.pattern.get(), 0)
 
-        def ls(oid):
-            if self.root().session().cmds.Flow.is_map(oid):
-                return self.root().session().cmds.Flow.get_mapped_oids(oid)
-
-            return list(map(
-                lambda rel: rel[0],
-                self.root().session().cmds.Flow.ls(oid)[0]
-            ))
+        objects = [self.root().get_object(oid) for oid in self._oids]
+        processed_oids = []
         
-        oids = [splitted_oids[0]]
+        for o in objects:
+            oid = o.oid()
 
-        for splitted_oid in splitted_oids[1:]:
-            updated_oids = []
-            if splitted_oid.startswith("/"):
-                splitted_oid = splitted_oid[1:]
+            if isinstance(o, Revision):
+                requested_site_name = self.requested_site_name.get()
+                requesting_site_name = self.requesting_site_name.get()
+
+                if not requesting_site_name in o.request_as.requesting_site_name.choices():
+                    oid += f" <font color='red'>Available on {requesting_site_name}</font>"
+                elif not requested_site_name in o.request_as.requested_site_name.choices():
+                    oid += f" <font color='red'>Not available on {requested_site_name}</font>"
+                else:
+                    o.request_as.run(None)
+                    oid += " <font color='green'>OK</font>"
+            else:
+                oid += " <font color='red'>Not a file revision</font>"
             
-            for oid in oids:
-                if oid.endswith("/"):
-                    oid = oid[:-1]
-                if not self.root().session().cmds.Flow.exists(oid):
-                    msg = self.message.get()
-                    msg += (
-                        "<font color=#D5000D>"
-                        f"Oid <b>{oid}</b> does not exist"
-                        "</font>"
-                    )
-                    self.message.set(msg)
-                    return self.get_result(close=False)
-
-                children = ls(oid)
-                
-                for child in children:
-                    updated_oids.append(child + "/" + splitted_oid)
-            
-            oids = updated_oids
-
-        processed_revisions = 0
-
-        for revision_oid in oids:
-            if not self.root().session().cmds.Flow.exists(revision_oid):
-                self.root().session().log_error(f"Revision {revision_oid} does not exist")
-                continue
-
-            revision = self.root().get_object(revision_oid)
-            
-            try:
-                revision.request_as.requesting_site_name.set(requesting_site_name)
-                revision.request_as.requested_site_name.set(requested_site_name)
-            except ValueError:
-                name = revision.name()
-                self.root().session().log_error((
-                    f"Revision {name} is either already available on requesting site {requesting_site_name} "
-                    f"or not available on requested site {requested_site_name}"
-                ))
-                continue
-
-            processed_revisions += 1
-            revision.request_as.run(None)
+            processed_oids.append(oid)
         
-        msg = "<h3>Request revisions</h3>"
-        msg += f"<b>{processed_revisions}</b> revision(s) processed."
-        self.message.set(msg)
+        self.revision_oids.set("<br>".join(processed_oids))
 
         return self.get_result(close=False)
 
