@@ -133,7 +133,7 @@ class Department(flow.Object):
     create_defaults = flow.Child(CreateDefaultFileSystemItemsAction).ui(
         label="Create default files"
     )
-    path = flow.Computed(cached=True)
+    path = flow.Computed(cached=True).ui(hidden=True)
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
