@@ -10,6 +10,7 @@ import re
 import hashlib
 import timeago
 import zipfile
+import fnmatch
 
 import kabaret.app.resources as resources
 from kabaret import flow
@@ -23,6 +24,26 @@ from .runners import LaunchSessionWorker, CHOICES, CHOICES_ICONS
 
 pyversion = sys.version_info
 
+
+class CreateWorkingCopyBaseAction(flow.Action):
+
+    _file = flow.Parent()
+
+    def _get_file(self):
+        return self._file
+
+    def allow_context(self, context):
+        settings = self.root().project().admin.project_settings
+        patterns = settings.non_editable_files.get().split(",")
+
+        for pattern in patterns:
+            pattern = pattern.encode('unicode-escape').decode().replace(" ", "")
+            if fnmatch.fnmatch(self._get_file().display_name.get(), pattern):
+                return False
+        
+        return True
+
+
 class RevisionsChoiceValue(flow.values.ChoiceValue):
 
     STRICT_CHOICES = False
@@ -44,10 +65,13 @@ class RevisionsChoiceValue(flow.values.ChoiceValue):
         self.set(name)
 
 
-class CreateWorkingCopyFromRevision(flow.Action):
+class CreateWorkingCopyFromRevision(CreateWorkingCopyBaseAction):
 
     _revision = flow.Parent()
 
+    def _get_file(self):
+        return self._revision._file
+
     def get_buttons(self):
         msg = "<h3>Create a working copy</h3>"
 
@@ -141,6 +165,116 @@ class CheckRevisionHash(flow.Action):
         return self.get_result(close=False)
 
 
+class PublishFileAction(LaunchSessionWorker):
+
+    ICON = ("icons.libreflow", "publish")
+
+    _file = flow.Parent()
+    _department = flow.Parent(3)
+
+    comment = flow.SessionParam("")
+    upload_after_publish = flow.BoolParam(False)
+
+    def get_buttons(self):
+        self.message.set("<h3>Publish</h3>")
+
+        if self._file.is_locked() and not self._file.is_locked(by_current_user=True):
+            msg = "<h3>This file is already used by %s</h3>" % self._file.locked_by()
+            msg += (
+                "You can't publish your changes while the file is used by someone else."
+            )
+            self.message.set(msg)
+
+            return ["Cancel"]
+
+        return ["Keep editing", "Unlock", "Cancel"]
+
+    def allow_context(self, context):
+        return context and self._file.has_working_copy(True)
+    
+    def launcher_oid(self):
+        return self.oid()
+
+    def launcher_exec_func_name(self):
+        return "upload_published_revision"
+    
+    def upload_published_revision(self):
+        head = self._file.get_head_revision()
+        current_site = self.root().project().get_current_site()
+
+        upload_job = current_site.get_queue().submit_job(
+            emitter_oid=head.oid(),
+            user=self.root().project().get_user(),
+            studio=current_site.name(),
+            job_type='Upload',
+            init_status='WAITING'
+        )
+        process_jobs_action = self.root().project().admin.process_jobs
+        process_jobs_action._process(upload_job)
+
+    def run(self, button):
+        if button == "Cancel":
+            return
+
+        self._file.lock()
+        keep_editing = (button == "Keep editing")
+        published_revision = self._file.publish(
+            comment=self.comment.get(),
+            keep_editing=keep_editing,
+        )
+
+        if not keep_editing:
+            self._file.set_current_user_on_revision(published_revision.name())
+            self._file.unlock()
+
+        if self._department.auto_current.get():
+            published_revision.make_current.run(None)
+
+        self._file.touch()
+
+        if self.upload_after_publish.get():
+            super(PublishFileAction, self).run(None)
+
+
+class PublishFileFromWorkingCopy(PublishFileAction):
+
+    _revision = flow.Parent()
+    _file = flow.Parent(4)
+    _department = flow.Parent(6)
+
+    def allow_context(self, context):
+        return context and self._revision.is_working_copy(from_current_user=True)
+
+    def run(self, button):
+        if button == "Cancel":
+            return
+        
+        # Store grand-parents to keep access to them after revision is removed
+        file = self._file
+        department = self._department
+        upload_after_publish = self.upload_after_publish.get()
+
+        file.lock()
+        keep_editing = (button == "Keep editing")
+
+        published_revision = file.publish(
+            comment=self.comment.get(),
+            keep_editing=keep_editing,
+        )
+
+        if not keep_editing:
+            file.set_current_user_on_revision(published_revision.name())
+            file.unlock()
+
+        if department.auto_current.get():
+            published_revision.make_current.run(None)
+
+        file.touch()
+
+        if upload_after_publish:
+            super(PublishFileAction, self).run(None)
+
+
 class Revision(flow.Object):
 
     _revisions = flow.Parent()
@@ -158,6 +292,7 @@ class Revision(flow.Object):
 
     open = flow.Child(OpenRevision)
     make_current = flow.Child(MakeCurrentRevisionAction)
+    publish = flow.Child(PublishFileFromWorkingCopy)
     create_working_copy = flow.Child(CreateWorkingCopyFromRevision)
     request = flow.Child(Request)
     request_as = flow.Child(RequestAs)
@@ -437,7 +572,7 @@ class CreateTrackedFileAction(flow.Action):
         self._files.touch()
 
 
-class CreateWorkingCopyAction(flow.Action):
+class CreateWorkingCopyAction(CreateWorkingCopyBaseAction):
 
     _file = flow.Parent()
 
@@ -737,74 +872,6 @@ class OpenWithNotepadPPAction(OpenWithAction):
         return ["txt"]
 
 
-class PublishFileAction(LaunchSessionWorker):
-
-    ICON = ("icons.libreflow", "publish")
-
-    _file = flow.Parent()
-    _department = flow.Parent(3)
-
-    comment = flow.SessionParam("")
-    upload_after_publish = flow.BoolParam(False)
-
-    def get_buttons(self):
-        self.message.set("<h3>Publish</h3>")
-
-        if self._file.is_locked() and not self._file.is_locked(by_current_user=True):
-            msg = "<h3>This file is already used by %s</h3>" % self._file.locked_by()
-            msg += (
-                "You can't publish your changes while the file is used by someone else."
-            )
-            self.message.set(msg)
-
-            return ["Cancel"]
-
-        return ["Keep editing", "Unlock", "Cancel"]
-
-    def allow_context(self, context):
-        return context and self._file.has_working_copy(True)
-    
-    def launcher_oid(self):
-        return self.oid()
-
-    def launcher_exec_func_name(self):
-        return "upload_published_revision"
-    
-    def upload_published_revision(self):
-        head = self._file.get_head_revision()
-        current_site = self.root().project().get_current_site()
-
-        upload_job = current_site.get_queue().submit_job(
-            emitter_oid=head.oid(),
-            user=self.root().project().get_user(),
-            studio=current_site.name(),
-            job_type='Upload',
-            init_status='WAITING'
-        )
-        process_jobs_action = self.root().project().admin.process_jobs
-        process_jobs_action._process(upload_job)
-
-    def run(self, button):
-        if button == "Cancel":
-            return
-
-        self._file.lock()
-        keep_editing = button == "Keep editing"
-        published_revision = self._file.publish(self.comment.get(), keep_editing)
-
-        if not keep_editing:
-            self._file.set_current_user_on_revision(published_revision.name())
-            self._file.unlock()
-
-        if self._department.auto_current.get():
-            published_revision.make_current.run(None)
-
-        self._file.touch()
-
-        if self.upload_after_publish.get():
-            super(PublishFileAction, self).run(None)
-
-
 class MakeFileCurrentAction(flow.Action):
 
     _file = flow.Parent()
@@ -846,20 +913,11 @@ class LockAction(flow.Action):
 
     _file = flow.Parent()
 
-    def get_buttons(self):
-        if self._file.is_locked(by_current_user=True):
-            msg = "<h2>You already locked this file.</h2>"
-        else:
-            msg = (
-                "<h2>This file is already locked by <b>%s</b></h2>"
-                % self._file.locked_by()
-            )
-        self.message.set(msg)
-
-        return []
-
+    def allow_context(self, context):
+        return context and not self._file.is_locked()
+    
     def needs_dialog(self):
-        return self._file.is_locked()
+        return False
 
     def run(self, button):
         self._file.lock()
@@ -872,30 +930,13 @@ class UnlockAction(flow.Action):
 
     _file = flow.Parent()
 
-    def get_buttons(self):
-        msg = (
-            "<h2>This file is already locked by <b>%s</b></h2>\n"
-            % self._file.locked_by()
-        )
-        msg += "You can't unlock a file locked by another user."
-        self.message.set(msg)
-
-        return ["Cancel"]
-
+    def allow_context(self, context):
+        return self._file.is_locked(by_current_user=True)
+    
     def needs_dialog(self):
-        current_user = self.root().project().get_user()
-        is_admin = self.root().project().is_admin(current_user)
-
-        return (
-            self._file.is_locked()
-            and not self._file.is_locked(by_current_user=True)
-            and not is_admin
-        )
+        return False
 
     def run(self, button):
-        if button == "Cancel":
-            return
-
         self._file.unlock()
         self._file._map.touch()
 
@@ -1217,23 +1258,20 @@ class TrackedFile(File):
     current_user_sees = flow.Computed()
 
     show_history = flow.Child(GotoHistory)
+    publish_action = flow.Child(PublishFileAction).ui(label="Publish")
     create_working_copy_action = flow.Child(CreateWorkingCopyAction).ui(
         label="Create working copy"
     )
-    publish_action = flow.Child(PublishFileAction).ui(label="Publish")
+    open = flow.Child(OpenTrackedFileAction)
+    see_revision = flow.Child(SeeRevisionAction).ui(label="See revision...")
+    reveal = flow.Child(RevealInExplorer).ui(label="Reveal in explorer")
     make_current_action = flow.Child(MakeFileCurrentAction).ui(
         label="Set last as current"
     )
-    open = flow.Child(OpenTrackedFileAction)
-    see_revision = flow.Child(SeeRevisionAction).ui(label="See revision...")
     lock_action = flow.Child(LockAction).ui(label="Lock")
     unlock_action = flow.Child(UnlockAction).ui(label="Unlock")
     search_existing_revisions = flow.Child(SearchExistingRevisions)
     request = flow.Child(RequestTrackedFileAction)
-    
-    with flow.group("Playblast"):
-        render_playblast = flow.Child(RenderBlenderPlayblastAction)
-        publish_and_render_playblast = flow.Child(PublishAndRenderPlayblastAction)
 
     with flow.group("Open with"):
         open_with_blender = flow.Child(OpenWithBlenderAction).ui(label="Blender")
@@ -1241,6 +1279,18 @@ class TrackedFile(File):
         open_with_vscodium = flow.Child(OpenWithVSCodiumAction).ui(label="VSCodium")
         open_with_notepadpp = flow.Child(OpenWithNotepadPPAction).ui(label="Notepad++")
 
+    with flow.group("Advanced"):
+        create_working_copy_from_file = flow.Child(None).ui(
+            label="Create working copy from another file"
+        )
+        publish_into_file = flow.Child(None).ui(
+            label="Publish to another file"
+        )
+    
+    with flow.group("Advanced/Playblast"):
+        render_playblast = flow.Child(RenderBlenderPlayblastAction)
+        publish_and_render_playblast = flow.Child(PublishAndRenderPlayblastAction)
+
     def get_name(self):
         return "%s_%s" % (self.complete_name.get(), self.format.get())
 
@@ -1354,7 +1404,7 @@ class TrackedFile(File):
             else:
                 return seen.comment.get()
 
-    def create_working_copy(self, reference_name=None, user_name=None):
+    def create_working_copy(self, reference_name=None, source_path=None, user_name=None):
         if user_name is None:
             user_name = self.root().project().get_user()
 
@@ -1390,24 +1440,26 @@ class TrackedFile(File):
 
         working_copy_path = working_copy.get_path()
 
-        if reference_name is None:
-            # Create working copy from scratch
-            file_path = self.get_template_path()
-            self.root().session().log_debug(
-                "Copy template {} to {}".format(file_path, working_copy_path)
-            )
-        else:
-            reference = self.get_revision(reference_name)
-            file_path = reference.get_path()
+        # If source path is given, ignore reference revision
+        if not source_path:
+            if reference_name is None:
+                # Create working copy from scratch
+                source_path = self.get_template_path()
+                self.root().session().log_debug(
+                    "Copy template {} to {}".format(source_path, working_copy_path)
+                )
+            else:
+                reference = self.get_revision(reference_name)
+                source_path = reference.get_path()
 
-            self.root().session().log_debug(
-                "Copy current revision {} to {}".format(
-                    file_path, working_copy_path
+                self.root().session().log_debug(
+                    "Copy current revision {} to {}".format(
+                        source_path, working_copy_path
+                    )
                 )
-            )
 
         try:
-            shutil.copy2(file_path, working_copy_path)
+            shutil.copy2(source_path, working_copy_path)
         except OSError:
             self.root().session().log_error(
                 "Copy of template '{}' failed\n\n==> TRACEBACK\n".format(file_path)
@@ -1416,14 +1468,12 @@ class TrackedFile(File):
 
         return working_copy
 
-    def publish(self, comment="", keep_editing=False):
+    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False):
         revisions = self.get_revisions()
-        head_revision = revisions.add()
+        head_revision = revisions.add(revision_name)
         head_revision.comment.set(comment)
         head_revision_path = head_revision.get_path()
         head_revision.set_sync_status("Available")
-        working_copy = self.get_working_copy()
-        working_copy_path = working_copy.get_path()
 
         self.root().session().log_debug(
             "Create head revision folder {}".format(head_revision.path.get())
@@ -1439,28 +1489,35 @@ class TrackedFile(File):
             )
             raise
 
-        if keep_editing:
-            self.root().session().log_debug(
-                "Copy working copy {} to {}".format(
-                    working_copy_path, head_revision_path
-                )
-            )
-
-            shutil.copy2(working_copy_path, head_revision_path)
+        # If source path is given, ignore working copy
+        if source_path:
+            shutil.copy2(source_path, head_revision_path)
         else:
-            self.root().session().log_debug(
-                "Copy working copy {} to {}".format(
-                    working_copy_path, head_revision_path
+            working_copy = self.get_working_copy()
+            working_copy_path = working_copy.get_path()
+
+            if keep_editing:
+                self.root().session().log_debug(
+                    "Copy working copy {} to {}".format(
+                        working_copy_path, head_revision_path
+                    )
                 )
-            )
-            self.root().session().log_debug(
-                "Remove working copy folder {}".format(working_copy.path.get())
-            )
 
-            shutil.move(working_copy_path, head_revision_path)
-            shutil.rmtree(working_copy.path.get())
+                shutil.copy2(working_copy_path, head_revision_path)
+            else:
+                self.root().session().log_debug(
+                    "Copy working copy {} to {}".format(
+                        working_copy_path, head_revision_path
+                    )
+                )
+                self.root().session().log_debug(
+                    "Remove working copy folder {}".format(working_copy.path.get())
+                )
 
-            revisions.remove(working_copy.name())
+                shutil.move(working_copy_path, head_revision_path)
+                shutil.rmtree(working_copy.path.get())
+
+                revisions.remove(working_copy.name())
 
         revisions.touch()
 
@@ -1514,11 +1571,11 @@ class TrackedFile(File):
         except flow.exceptions.MappedNameError:
             return None
 
-    def get_head_revision(self):
+    def get_head_revision(self, sync_status=None):
         revisions = self.get_revisions()
 
         for revision in reversed(revisions.mapped_items()):
-            if not revision.is_working_copy():
+            if not revision.is_working_copy() and (not sync_status or revision.get_sync_status() == sync_status):
                 return revision
 
         return None
@@ -1606,6 +1663,283 @@ class TrackedFile(File):
             super(TrackedFile, self).compute_child_value(child_value)
 
 
+class FileRevisionNameChoiceValue(flow.values.ChoiceValue):
+
+    STRICT_CHOICES = False
+    action = flow.Parent()
+
+    def get_file(self):
+        return self.action._file
+
+    def choices(self):
+        source_file = self.get_file()
+        names = []
+        
+        if not source_file:
+            return names
+        
+        for rev in source_file.get_revisions().mapped_items():
+            if rev.get_sync_status() == "Available":
+                names.append(rev.name())
+        
+        return names
+    
+    def revert_to_default(self):
+        source_file = self.get_file()
+        if source_file:
+            head = source_file.get_head_revision(sync_status="Available")
+            self.set(head.name() if head else None)
+        else:
+            self.set(None)
+
+
+class FileRefRevisionNameChoiceValue(FileRevisionNameChoiceValue):
+
+    def get_file(self):
+        return self.action.source_file.get()
+
+
+class ResetRef(flow.Action):
+
+    _ref = flow.Parent()
+
+    def allow_context(self, context):
+        return context and context.endswith(".inline")
+    
+    def needs_dialog(self):
+        return False
+    
+    def run(self, button):
+        self._ref.set(None)
+        return self.get_result(refresh=True)
+
+
+class ResetableTrackedFileRef(flow.values.Ref):
+
+    SOURCE_TYPE = TrackedFile
+    reset = flow.Child(ResetRef)
+
+
+class PublishIntoFile(PublishFileAction):
+
+    source_file = flow.SessionParam("").ui(
+        editable=False,
+        tooltip="File to publish to.",
+    )
+    source_revision_name = flow.Param(None, FileRevisionNameChoiceValue).watched().ui(
+        label="Source revision"
+    )
+    target_file = flow.Connection(ref_type=ResetableTrackedFileRef).watched()
+    revision_name = flow.SessionParam("").watched()
+    comment = flow.SessionParam("")
+    upload_after_publish = flow.BoolParam(False)
+
+    def get_buttons(self):
+        self.message.set("<h2>Publish from an existing file</h2>")
+        self.source_file.set(self._file.display_name.get())
+        self.source_revision_name.revert_to_default()
+
+        if self._file.is_locked() and not self._file.is_locked(by_current_user=True):
+            msg = "<h2>This file is already used by %s</h2>" % self._file.locked_by()
+            msg += (
+                "You can't publish your changes while the file is used by someone else."
+            )
+            self.message.set(msg)
+
+            return ["Cancel"]
+
+        return ["Publish", "Cancel"]
+
+    def allow_context(self, context):
+        return None
+
+    def check_file(self, file):
+        expected_format = self._file.format.get()
+        msg = "<h2>Publish from an existing file</h2>"
+
+        if not file or file.format.get() != expected_format:
+            self.message.set((
+                f"{msg}<font color=#D5000D>"
+                f"Target file must be in {expected_format} format."
+                "</font>"
+            ))
+            return False
+        elif not self.source_revision_name.choices():
+            self.message.set((
+                f"{msg}<font color=#D5000D>"
+                f"Target file has no revision available on current site."
+                "</font>"
+            ))
+            return False
+        
+        return True
+    
+    def check_revision_name(self, name):
+        msg = "<h2>Publish from an existing file</h2>"
+        target_file = self.target_file.get()
+
+        if not self.check_file(target_file):
+            return False
+
+        if target_file.has_revision(name):
+            msg += (
+                "<font color=#D5000D>"
+                f"Target file already has a revision {name}."
+                "</font>"
+            )
+            self.message.set(msg)
+
+            return False
+        
+        self.message.set(msg)
+
+        return True
+    
+    def child_value_changed(self, child_value):
+        self.message.set("<h2>Publish from an existing file</h2>")
+
+        if child_value is self.target_file:
+            self.check_file(self.target_file.get())
+            self.check_revision_name(self.source_revision_name.get())
+        elif child_value is self.source_revision_name:
+            self.revision_name.set(self.source_revision_name.get())
+            self.comment.set("Created from %s (%s)" % (
+                self._file.display_name.get(),
+                self.source_revision_name.get(),
+            ))
+        elif child_value is self.revision_name:
+            revision_name = self.revision_name.get()
+            self.check_revision_name(revision_name)
+
+            self.comment.set("Created from %s (%s)" % (
+                self._file.display_name.get(),
+                revision_name,
+            ))
+
+    def run(self, button):
+        if button == "Cancel":
+            return
+
+        target_file = self.target_file.get()
+
+        # Check source file
+        if not self.check_file(target_file):
+            return self.get_result(close=False)
+        
+        revision_name = self.revision_name.get()
+        
+        # Check choosen revision name
+        if not self.check_revision_name(revision_name):
+            return self.get_result(close=False)
+        
+        source_revision = self._file.get_revision(revision_name)
+        
+        # Publish in target file
+        target_file.lock()
+
+        publication = target_file.publish(
+            revision_name=revision_name,
+            source_path=source_revision.get_path(),
+            comment=self.comment.get(),
+        )
+        target_file.unlock()
+
+        if self._department.auto_current.get():
+            publication.make_current.run(None)
+
+        target_file._map.touch()
+
+        if self.upload_after_publish.get():
+            super(PublishFileAction, self).run(None)
+
+
+class CreateWorkingCopyFromFile(CreateWorkingCopyBaseAction):
+
+    _file = flow.Parent()
+    source_file = flow.Connection(ref_type=ResetableTrackedFileRef).watched()
+    source_revision_name = flow.Param(None, FileRefRevisionNameChoiceValue).ui(
+        label="Source revision"
+    )
+    target_file = flow.SessionParam("").ui(
+        editable=False,
+        tooltip="File in which the working copy will be created.",
+    )
+
+    def get_buttons(self):
+        msg = "<h2>Create working copy from another file</h2>"
+        self.target_file.set(self._file.display_name.get())
+
+        if self._file.has_working_copy(from_current_user=True):
+            msg += (
+                "<font color=#D66700>"
+                "You already have a working copy on %s. "
+                "Creating a working copy will overwrite the current one."
+                "</font>" % self._file.display_name.get()
+            )
+        
+        self.message.set(msg)
+
+        return ["Create", "Cancel"]
+    
+    def child_value_changed(self, child_value):
+        if child_value is self.source_file:
+            self.check_file(self.source_file.get())
+
+            self.source_revision_name.touch()
+            self.source_revision_name.revert_to_default()
+
+    def check_file(self, file):
+        expected_format = self._file.format.get()
+        msg = "<h2>Create working copy from another file</h2>"
+
+        if not file or file.format.get() != expected_format:
+            self.message.set((
+                f"{msg}<font color=#D5000D>"
+                f"Source file must be in {expected_format} format."
+                "</font>"
+            ))
+            return False
+        elif not self.source_revision_name.choices():
+            self.message.set((
+                f"{msg}<font color=#D5000D>"
+                f"Source file has no revision available on current site."
+                "</font>"
+            ))
+            return False
+
+        if self._file.has_working_copy(from_current_user=True):
+            msg += (
+                "<font color=#D66700>"
+                "You already have a working copy on %s. "
+                "Creating a working copy will overwrite the current one."
+                "</font>" % self._file.display_name.get()
+            )
+
+        self.message.set(msg)
+        
+        return True
+    
+    def run(self, button):
+        if button == "Cancel":
+            return
+
+        source_file = self.source_file.get()
+
+        if not self.check_file(source_file):
+            return self.get_result(close=False)
+        
+        source_revision = source_file.get_revision(self.source_revision_name.get())
+        working_copy = self._file.create_working_copy(source_path=source_revision.get_path())
+
+        self._file.set_current_user_on_revision(working_copy.name())
+        self._file.touch()
+        self._file.get_revisions().touch()
+
+
+TrackedFile.create_working_copy_from_file.set_related_type(CreateWorkingCopyFromFile)
+TrackedFile.publish_into_file.set_related_type(PublishIntoFile)
+
+
 class ClearFileSystemMapAction(ClearMapAction):
     def run(self, button):
         for item in self._map.mapped_items():
@@ -1767,35 +2101,42 @@ class TrackedFolder(TrackedFile):
 
         return working_copy
 
-    def publish(self, comment="", keep_editing=False):
+    def publish(self, revision_name=None, source_path=None, comment="", keep_editing=False):
         revisions = self.get_revisions()
-        head_revision = revisions.add()
+        head_revision = revisions.add(revision_name)
         head_revision.comment.set(comment)
         head_revision.set_sync_status("Available")
 
-        working_copy = self.get_working_copy()
-        working_copy_path = working_copy.get_path()
-        head_revision_path = head_revision.get_path()
-
-        self.root().session().log_debug(
-            "Create head revision folder %s" % head_revision.path.get()
-        )
-
-        if keep_editing:
-            copy_function = shutil.copy2
+        if source_path:
+            shutil.copytree(
+                source_path,
+                head_revision.path.get(),
+                ignore=shutil.ignore_patterns("*.zip"),
+            )
         else:
-            copy_function = shutil.move
-            revisions.remove(working_copy.name())
-        
-        # Move working copy files to head folder
-        shutil.copytree(
-            working_copy.path.get(),
-            head_revision.path.get(),
-            ignore=shutil.ignore_patterns("*.zip"),
-            copy_function=copy_function
-        )
-        if not keep_editing:
-            shutil.rmtree(working_copy.path.get())
+            working_copy = self.get_working_copy()
+            working_copy_path = working_copy.get_path()
+            head_revision_path = head_revision.get_path()
+
+            self.root().session().log_debug(
+                "Create head revision folder %s" % head_revision.path.get()
+            )
+
+            if keep_editing:
+                copy_function = shutil.copy2
+            else:
+                copy_function = shutil.move
+                revisions.remove(working_copy.name())
+            
+            # Move working copy files to head folder
+            shutil.copytree(
+                working_copy.path.get(),
+                head_revision.path.get(),
+                ignore=shutil.ignore_patterns("*.zip"),
+                copy_function=copy_function
+            )
+            if not keep_editing:
+                shutil.rmtree(working_copy.path.get())
         
         # Pack files in head zip file
         files = os.listdir(head_revision.path.get())
