@@ -10,7 +10,7 @@ from kabaret.subprocess_manager.runner_factory import Runner, logger
 from kabaret.subprocess_manager.flow import RunAction
 
 
-CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip"]
+CHOICES = ["blend", "kra", "png", "jpg", "txt", "abc", "mov", "psd", "aep", "zip", "mp4", "fbx", "ai"]
 CHOICES_ICONS = {
             "blend": ("icons.libreflow", "blender"),
             "kra": ("icons.libreflow", "krita"),
@@ -21,7 +21,10 @@ CHOICES_ICONS = {
             "aep": ("icons.libreflow", "afterfx"),
             "psd": ("icons.flow", "photoshop"),
             "mov": ("icons.flow", "quicktime"),
+            "ai":  ("icons.libreflow", "illustrator"),
             "zip": ("icons.flow", "file"),
+            "mp4": ("icons.gui", "youtube-logo"),
+            "fbx": ("icons.libreflow", "fbx"),
         }
 
 
