@@ -17,7 +17,7 @@ from libreflow.baseflow.runners import BaseRunner, PythonRunner
 
 
 class StaticSiteTypeChoices(flow.values.ChoiceValue):
-    CHOICES = ['Studio', 'User', 'Exchange']
+    CHOICES = ['Studio', 'User']
 
 
 class StaticSiteSyncStatusChoices(flow.values.ChoiceValue):
@@ -44,7 +44,7 @@ class Job(flow.Object):
     status = flow.Param('WAITING', JobStatus)
     priority = flow.IntParam(50)
     emitter_oid = flow.Param()
-    date = flow.Param().ui(editor='datetime')
+    date = flow.Param("").ui(editor='datetime')
     date_end = flow.Param("").ui(editor='datetime')
     is_archived = flow.BoolParam()
     requested_by_user = flow.Param()
@@ -116,7 +116,7 @@ class JobQueue(flow.Map):
             emitter_oid,
             user,
             studio,
-            date_end=None,
+            date_end=-1,
             job_type='Download',
             init_status='WAITING',
             priority=50
@@ -130,19 +130,19 @@ class JobQueue(flow.Map):
         )
         job = self.insert(name, priority)
 
-        self.root().session().log_info("Submitted job %s" % job)
-
         job.type.set(job_type)
         job.status.set(init_status)
         job.priority.set(priority)
         job.emitter_oid.set(emitter_oid)
-        job.date.set(datetime.datetime.fromtimestamp(time.time()).ctime())
+        job.date.set(time.time())
         job.date_end.set(date_end)
         job.requested_by_user.set(user)
         job.requested_by_studio.set(studio)
         job.is_archived.set(False)
         job.log.set('?')
 
+        self.root().session().log_info("Submitted job %s" % job)
+
         self.touch()
 
         return job
@@ -161,8 +161,8 @@ class JobQueue(flow.Map):
         row["Type"] = item.type.get()
         row["Status"] = item.status.get()
         row["Priority"] = item.priority.get()
-        row["Emitted on"] = item.date.get()
-        row["Expires on"] = item.date_end.get()
+        row["Emitted on"] = datetime.datetime.fromtimestamp(item.date.get()).ctime()
+        row["Expires on"] = "Never" if item.date_end.get() == -1 else item.date_end.get()
         row["Requested by (site/user)"] = "%s/%s" % (
             item.requested_by_studio.get(),
             item.requested_by_user.get()
@@ -181,7 +181,7 @@ class ProcessJobs(flow.Action):
     
     def _get_jobs(self):
         current_site = self.root().project().get_current_site()
-        return current_site.queue.jobs()
+        return current_site.get_jobs()
     
     def run(self, button):
         for job in self._get_jobs():
@@ -218,11 +218,11 @@ class MinioUploadFile(flow.Object):
                 server_path
             )
         )
-        current_site = self.root().project().get_current_site()
+        exchange_site = self.root().project().get_exchange_site()
         minioClient = Minio(
-            current_site.exchange_server_url.get(),
-            access_key=current_site.exchange_server_login.get(),
-            secret_key=current_site.exchange_server_password.get(),
+            exchange_site.server_url.get(),
+            access_key=exchange_site.server_login.get(),
+            secret_key=exchange_site.server_password.get(),
             secure=True
         )
 
@@ -242,11 +242,11 @@ class MinioDownloadFile(flow.Object):
                 local_path
             )
         )
-        current_site = self.root().project().get_current_site()
+        exchange_site = self.root().project().get_exchange_site()
         minioClient = Minio(
-            current_site.exchange_server_url.get(),
-            access_key=current_site.exchange_server_login.get(),
-            secret_key=current_site.exchange_server_password.get(),
+            exchange_site.server_url.get(),
+            access_key=exchange_site.server_login.get(),
+            secret_key=exchange_site.server_password.get(),
             secure=True
         )
 
@@ -270,7 +270,7 @@ class Synchronize(ProcessJobs):
     def _get_jobs(self):
         # Get current site's waiting jobs only
         current_site = self.root().project().get_current_site()
-        return current_site.queue.jobs(status="WAITING")
+        return current_site.get_jobs(status="WAITING")
 
     def _get_sync_status(self, revision_oid, site_name=None, site_type="Studio"):
         if not site_name and site_type == "Exchange":
@@ -340,6 +340,7 @@ class Synchronize(ProcessJobs):
 
             self.uploader.upload(local_path, server_path)
             self._set_sync_status(job.emitter_oid.get(), "Available", site_type="Exchange")
+            self._touch(job.emitter_oid.get() + "/..")
         elif job.type.get() == "Download":
             sync_status = self._get_sync_status(job.emitter_oid.get(), site_type="Exchange")
             
@@ -354,6 +355,7 @@ class Synchronize(ProcessJobs):
             self.downloader.download(server_path, local_path)
             self._set_sync_status(job.emitter_oid.get(), "Available")
             self._touch(job.emitter_oid.get() + "/..")
+            self._touch(job.emitter_oid.get() + "/../../../..")
         
         job.status.set("PROCESSED")
         
@@ -416,12 +418,11 @@ class RequestAs(flow.Action):
         requesting_site_name = self.requesting_site_name.get()
 
         # Get requesting and requested sites
-        sites = self.root().project().admin.sites
-        requested_site = sites[self.requested_site_name.get()]
+        sites = self.root().project().get_working_sites()
         requesting_site = sites[requesting_site_name]
         
         # Add a download job for the requesting site
-        requesting_site.queue.submit_job(
+        requesting_site.get_queue().submit_job(
             job_type="Download",
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
@@ -433,19 +434,20 @@ class RequestAs(flow.Action):
         self._revision._revisions.touch()
         
         # Check if the version is not available on the exchange server
-        for site_name in self._revision.sync.mapped_names():
-            site = sites[site_name]
-            site_status = self._revision.sync[site_name]
+        exchange_site = self.root().project().get_exchange_site()
+        revision_status = self._revision.get_sync_status(exchange=True)
+
+        if revision_status == "Available":
+            self.root().session().log_warning(
+                "Revision already available on exchange server"
+            )
+            self.root().project().synchronization.touch()
+            return self.get_result()
+
+        requested_site = sites[self.requested_site_name.get()]
 
-            if site.site_type.get() == "Exchange" and site_status.status.get() == "Available":
-                self.root().session().log_warning(
-                    "Revision already available on exchange server"
-                )
-                self.root().project().synchronization.touch()
-                return self.get_result()
-            
         # Check if the source version upload is not already requested
-        for job in requested_site.queue.jobs(type="Upload", status="WAITING"):
+        for job in requested_site.get_jobs(type="Upload", status="WAITING"):
             if job.emitter_oid.get() == self._revision.oid():
                 self.root().session().log_warning(
                     "Revision already requested for upload in source site"
@@ -454,7 +456,7 @@ class RequestAs(flow.Action):
                 return self.get_result()
         
         # Add an upload job for the requested site
-        requested_site.queue.submit_job(
+        requested_site.get_queue().submit_job(
             job_type="Upload",
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
@@ -475,7 +477,7 @@ class Request(RequestAs):
 
     def get_buttons(self):
         self.requesting_site_name.set(
-            self.root().project().admin.current_site_name.get()
+            self.root().project().get_current_site().name()
         )
 
         return super(Request, self).get_buttons()
@@ -493,33 +495,204 @@ class ResetJobStatuses(flow.Action):
     status = flow.Param("WAITING", JobStatus)
 
     def run(self, button):
-        for job in self._site.queue.mapped_items():
+        for job in self._site.get_jobs():
             job.status.set(self.status.get())
         
-        self._site.queue.touch()
+        self._site.get_queue().touch()
 
 
-class ClearQueue(flow.Action):
+class CreateSite(flow.Action):
+
+    ICON = ("icons.gui", "plus-sign-in-a-black-circle")
+
+    _site_map = flow.Parent()
+    site_name = flow.SessionParam("").ui(label="Name")
+
+    def get_buttons(self):
+        self.message.set("<h2>Create a site</h2>")
+        return ["Create", "Cancel"]
+    
+    def run(self, button):
+        if button == "Cancel":
+            return
+        
+        site_name = self.site_name.get()
+
+        if not site_name or self._site_map.has_mapped_name(site_name):
+            if not site_name:
+                msg = "Site name must not be empty."
+            else:
+                msg = f"Site {site_name} already exists."
+
+            self.message.set((
+                "<h2>Create a site</h2>"
+                f"<font color=#D5000D>{msg}</font>"
+            ))
+            
+            return self.get_result(close=False)
+
+        site = self._site_map.add(site_name)
+        self._site_map.touch()
+
+        return self.get_result(next_action=site.configuration.oid())
+
+
+class ConfigureSite(flow.Action):
+
+    _site_map = flow.Parent(2)
+    _site = flow.Parent()
+    short_name = flow.SessionParam("").ui(label="Short name")
+    description = flow.SessionParam("")
+
+    def get_buttons(self):
+        self.message.set(
+            "<h2>Site <font color=#fff>%s</font></h2>" % self._site.name()
+        )
+        self._fill_action_fields()
+
+        return ["Configure", "Cancel"]
+
+    def _configure_site(self, site):
+        '''
+        This can be used by subclass to configure a mapped site.
+
+        Default is to set site's short name and description.
+        '''
+        self._site.short_name.set(self.short_name.get())
+        self._site.description.set(self.description.get())
+    
+    def _fill_action_fields(self):
+        '''
+        This can be used by subclass to fill action's parameters when
+        dialog is displayed (e.g. to automatically show site parameters).
+
+        Default is to fill site's short name and description.
+        '''
+        self.short_name.set(self._site.short_name.get())
+        self.description.set(self._site.description.get())
+    
+    def allow_context(self, context):
+        # Hide in map item submenus
+        return False
+    
+    def run(self, button):
+        if button == "Cancel":
+            return
+        
+        self._configure_site(self._site)
+        self._site.configured.touch()
+        self._site_map.touch()
+
+
+class ComputedBoolValue(flow.values.ComputedValue):
+
+    DEFAULT_EDITOR = 'bool'
+
+
+class Site(flow.Object):
+
+    short_name = flow.Param("")
+    description = flow.Param("")
+    # Define conditions required for a site to be considered as correctly configured
+    configured = flow.Computed(
+        cached=True,
+        store_value=True,
+        computed_value_type=ComputedBoolValue
+    )
+    configuration = flow.Child(ConfigureSite)
+
+    def compute_child_value(self, child_value):
+        if child_value is self.configured:
+            self.configured.set(True)
+
+
+class SiteMap(flow.Map):
+
+    create_site = flow.Child(CreateSite)
+
+    @classmethod
+    def mapped_type(cls):
+        return Site
+
+    def mapped_names(self, page_num=0, page_size=None):
+        return ["default"] + super(SiteMap, self).mapped_names(page_num, page_size)
+    
+    def columns(self):
+        return ["Site"]
+    
+    def short_names(self):
+        return [s.short_name.get() for s in self.mapped_items()]
+    
+    def _get_mapped_item_type(self, mapped_name):
+        if mapped_name == "default":
+            return self.mapped_type()
+
+        return super(SiteMap, self)._get_mapped_item_type(mapped_name)
+    
+    def _fill_row_cells(self, row, item):
+        row["Site"] = item.name()
+        if not item.configured.get():
+            row["Site"] += " ⚠️"
+    
+    def _fill_row_style(self, style, item, row):
+        style["activate_oid"] = item.configuration.oid()
+
+
+class ConfigureWorkingSite(ConfigureSite):
+
+    site_type = flow.Param("Studio", StaticSiteTypeChoices)
+
+    root_windows_folder = flow.SessionParam("")
+    root_linux_folder = flow.SessionParam("")
+    root_darwin_folder = flow.SessionParam("")
+    
+    sync_dl_max_connections = flow.SessionParam(1)
+    sync_up_max_connections = flow.SessionParam(1)
+    
+    def _fill_action_fields(self):
+        super(ConfigureWorkingSite, self)._fill_action_fields()
+        self.site_type.set(self._site.site_type.get())
+        self.root_windows_folder.set(self._site.root_windows_folder.get())
+        self.root_linux_folder.set(self._site.root_linux_folder.get())
+        self.root_darwin_folder.set(self._site.root_darwin_folder.get())
+        self.sync_dl_max_connections.set(self._site.sync_dl_max_connections.get())
+        self.sync_up_max_connections.set(self._site.sync_up_max_connections.get())
+
+    def _configure_site(self, site):
+        super(ConfigureWorkingSite, self)._configure_site(site)
+        site.site_type.set(self.site_type.get())
+        site.root_windows_folder.set(self.root_windows_folder.get())
+        site.root_linux_folder.set(self.root_linux_folder.get())
+        site.root_darwin_folder.set(self.root_darwin_folder.get())
+        site.sync_dl_max_connections.set(self.sync_dl_max_connections.get())
+        site.sync_up_max_connections.set(self.sync_up_max_connections.get())
+
+
+class Queue(flow.Object):
+
+    job_list = flow.Child(JobQueue).ui(label="Jobs", expanded=True)
+
+
+class GotoQueue(flow.Action):
+
     _site = flow.Parent()
 
     def needs_dialog(self):
         return False
     
     def run(self, button):
-        self._site.queue.clear()
-        self._site.queue.touch()
+        return self.get_result(goto=self._site.queue.oid())
 
 
-class Site(flow.Object):
-    short_name = flow.Param("")
-    description = flow.Param("")
-    request_files_from_anywhere = flow.BoolParam(False).ui(tooltip="Allowed to request files for any site")
-
+class WorkingSite(Site):
+    
     site_type = flow.Param("Studio", StaticSiteTypeChoices)
-
-    exchange_server_url = flow.Param("http://")
-    exchange_server_login = flow.Param("")
-    exchange_server_password = flow.Param("").ui(editor='password')
+    request_files_from_anywhere = flow.BoolParam(False).ui(
+        tooltip=(
+            "Allow the site to request files for any other site. "
+            "Temporary option as long as synchronisation is manual."
+        )
+    )
 
     root_folder = flow.Computed(cached=True)
     root_windows_folder = flow.Param()
@@ -529,11 +702,12 @@ class Site(flow.Object):
     sync_dl_max_connections = flow.IntParam(1)
     sync_up_max_connections = flow.IntParam(1)
 
-    queue = flow.Child(JobQueue).ui(expanded=True)
+    queue = flow.Child(Queue)
 
-    remove_site = flow.Child(RemoveGenericAction)
-    reset_statuses = flow.Child(ResetJobStatuses)
-    clear_queue = flow.Child(ClearQueue)
+    configuration = flow.Child(ConfigureWorkingSite)
+    goto_queue = flow.Child(GotoQueue).ui(
+        label="Show job queue"
+    )
 
     def compute_child_value(self, child_value):
         if child_value is self.root_folder:
@@ -554,57 +728,124 @@ class Site(flow.Object):
 
             child_value.set(root_dir)
 
-
-class Sites(ItemMap):
-    ICON = ('icons.gui', 'home')
-
-    create_site = flow.Child(CreateGenericAction)
-
-    def mapped_names(self, page_num=0, page_size=None):
-        return ["default"] + super(Sites, self).mapped_names(page_num, page_size)
+        elif child_value is self.configured:
+            self.configured.set(
+                (
+                    self.root_windows_folder.get()
+                    or self.root_linux_folder.get()
+                    or self.root_darwin_folder.get()
+                )
+            )
     
-    def short_names(self):
-        return [s.short_name.get() for s in self.mapped_items()]
+    def get_queue(self):
+        return self.queue.job_list
+    
+    def get_jobs(self, type=None, status=None):
+        return self.get_queue().jobs(type=type, status=status)
 
-    def _configure_child(self, child):
-        if child.name() == "default":
-            child.description.set("default_value")
-            child.site_type.set("Studio")
 
-    def _get_mapped_item_type(self, mapped_name):
-        if mapped_name == "default":
-            return self.mapped_type()
+class WorkingSites(SiteMap):
 
-        return super(Sites, self)._get_mapped_item_type(mapped_name)
+    ICON = ('icons.gui', 'home')
 
     @classmethod
     def mapped_type(cls):
-        return Site
+        return WorkingSite
 
-    def columns(self):
-        return ['Name', 'Waiting jobs (upload)', 'Waiting jobs (download)']
-
-    def _fill_row_cells(self, row, item):
-        row['Name'] = item.name()
-        row['Waiting jobs (upload)'] = len(item.queue.jobs(type="Upload", status="WAITING"))
-        row['Waiting jobs (download)'] = len(item.queue.jobs(type="Download", status="WAITING"))
+    def _configure_child(self, child):
+        if child.name() == "default":
+            child.short_name.set("dft")
+            child.description.set("Default working site")
+            child.site_type.set("Studio")
+        else:
+            super(WorkingSites, self)._configure_child(child)
 
     def _fill_row_style(self, style, item, row):
+        super(WorkingSites, self)._fill_row_style(style, item, row)
+
         if item.site_type.get() == "User":
             style['icon'] = ('icons.gui', 'user')
-        elif item.site_type.get() == "Studio":
-            style['icon'] = ('icons.gui', 'home')
-        elif item.site_type.get() == "Exchange":
-            style['icon'] = ('icons.gui', 'sitemap')
         else:
-            style['icon'] = ('icons.gui', 'vline')
+            style['icon'] = ('icons.gui', 'home')
+
+
+class ConfigureExchangeSite(ConfigureSite):
+
+    server_url = flow.SessionParam("http://")
+    server_login = flow.SessionParam("")
+    server_password = flow.SessionParam("").ui(editor='password')
+
+    def _fill_action_fields(self):
+        super(ConfigureExchangeSite, self)._fill_action_fields()
+        self.server_url.set(self._site.server_url.get())
+        self.server_login.set(self._site.server_login.get())
+        self.server_password.set(self._site.server_password.get())
+
+    def _configure_site(self, site):
+        super(ConfigureExchangeSite, self)._configure_site(site)
+        site.server_url.set(self.server_url.get())
+        site.server_login.set(self.server_login.get())
+        site.server_password.set(self.server_password.get())
+
+
+class ExchangeSite(Site):
+
+    ICON = ('icons.libreflow', 'exchange')
+
+    server_url = flow.Param("")
+    server_login = flow.Param("")
+    server_password = flow.Param("").ui(editor='password')
+
+    configuration = flow.Child(ConfigureExchangeSite)
+
+    def compute_child_value(self, child_value):
+        if child_value is self.configured:
+            self.configured.set(
+                (
+                    self.server_url.get()
+                    and self.server_login.get()
+                    and self.server_password.get()
+                )
+            )
+
+
+class ExchangeSites(SiteMap):
+    
+    ICON = ('icons.libreflow', 'exchange')
+
+    @classmethod
+    def mapped_type(cls):
+        return ExchangeSite
+
+    def mapped_names(self, page_num=0, page_size=None):
+        return ["default_exchange"] + super(SiteMap, self).mapped_names(page_num, page_size)
+
+    def _configure_child(self, child):
+        if child.name() == "default_exchange":
+            child.short_name.set("dftx")
+            child.description.set("Default exchange site")
+        else:
+            super(ExchangeSites, self)._configure_child(child)
+
+    def _get_mapped_item_type(self, mapped_name):
+        if mapped_name == "default_exchange":
+            return self.mapped_type()
+
+        return super(SiteMap, self)._get_mapped_item_type(mapped_name)
 
 
 class SyncSiteStatus(flow.Object):
     status = flow.Param("NotAvailable", StaticSiteSyncStatusChoices)
 
     def get_short_name(self):
-        site = self.root().project().admin.sites[self.name()]
+        working_sites = self.root().project().get_working_sites()
+        exchange_sites = self.root().project().get_exchange_sites()
+
+        if working_sites.has_mapped_name(self.name()):
+            site = working_sites[self.name()]
+        else:
+            site = exchange_sites[self.name()]
+
         return site.short_name.get()
 
 
@@ -616,15 +857,10 @@ class SyncMap(flow.DynamicMap):
         return SyncSiteStatus
 
     def mapped_names(self, page_num=0, page_size=None):
-        project = self.root().project()
-        project_sites = project.admin.sites
-
-        sites = []
-
-        for s in project_sites.mapped_items():
-            sites.append(s.name())
-
-        return sites
+        working_sites = self.root().project().get_working_sites()
+        exchange_sites = self.root().project().get_exchange_sites()
+        
+        return working_sites.mapped_names() + exchange_sites.mapped_names()
 
     def columns(self):
         return ['Name', 'Status']
