@@ -3,10 +3,6 @@ import json
 import getpass
 
 from kabaret import flow
-from .maputils import CreateGenericItemAction, ClearMapAction
-
-
-
 
 
 class MyBookmarks(flow.DynamicMap):
@@ -59,9 +55,12 @@ class UserProfile(flow.Object):
                 return
             
             # Check user file
-            user_file = os.path.join(self.root().project().get_user_folder(), "user.json")
-            if os.path.exists(user_file):
-                with open(user_file, "r") as f:
+            current_user_file = os.path.join(
+                self.root().project().project_settings_folder(),
+                "current_user.json"
+            )
+            if os.path.exists(current_user_file):
+                with open(current_user_file, "r") as f:
                     user_config = json.load(f)
                     child_value.set(user_config["username"])
                     return
@@ -248,7 +247,6 @@ class Users(flow.Map):
     ICON = ("icons.gui", "team")
 
     add_user_action = flow.Child(AddUserAction).ui(label="Add user")
-    clear_map = flow.Child(ClearMapAction)
 
     @classmethod
     def mapped_type(cls):
@@ -387,7 +385,7 @@ class UserEnvironment(flow.DynamicMap):
         return env.keys()
 
     def file_path(self):
-        return "%s/env.json" % self.root().project().get_user_folder()
+        return "%s/env.json" % self.root().project().user_settings_folder()
 
     def _configure_child(self, child):
         with open(self.file_path(), "r") as f:
